@@ -1,12 +1,13 @@
+#!/usr/bin/env python
+# PYTHON_ARGCOMPLETE_OK
+
 import argparse
 import logging
 import math
 import re
 import sys
-
-from simplyblock_core import cluster_ops, utils
-from simplyblock_core import kv_store
-from simplyblock_core import compute_node_ops as compute_ops
+import argcomplete
+from simplyblock_core import cluster_ops, utils, db_controller
 from simplyblock_core import storage_node_ops as storage_ops
 from simplyblock_core import mgmt_node_ops as mgmt_ops
 from simplyblock_core import constants
@@ -19,9 +20,7 @@ from simplyblock_core.models.pool import Pool
 class CLIWrapper:
 
     def __init__(self):
-        self.logger = logging.getLogger()
-        self.logger.setLevel(constants.LOG_LEVEL)
-        self.db_store = kv_store.KVStore()
+        self.logger = utils.get_logger()
         self.init_parser()
 
         #
@@ -63,29 +62,29 @@ class CLIWrapper:
 
         # delete storage node
         sub_command = self.add_sub_command(subparser, "delete", 'Delete storage node obj')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         # remove storage node
         sub_command = self.add_sub_command(subparser, "remove", 'Remove storage node')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--force-remove", help='Force remove all LVols and snapshots',
                                  dest='force_remove', required=False, action='store_true')
-        sub_command.add_argument("--force-migrate", help='Force migrate All LVols to other nodes',
-                                 dest='force_migrate', required=False, action='store_true')
+        # sub_command.add_argument("--force-migrate", help='Force migrate All LVols to other nodes',
+        #                          dest='force_migrate', required=False, action='store_true')
         # List all storage nodes
         sub_command = self.add_sub_command(subparser, "list", 'List storage nodes')
         sub_command.add_argument("--cluster-id", help='id of the cluster for which nodes are listed', dest='cluster_id')
         sub_command.add_argument("--json", help='Print outputs in json format', action='store_true')
 
         sub_command = self.add_sub_command(subparser, "get", 'Get storage node info')
-        sub_command.add_argument("id", help='UUID of storage node')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         # Restart storage node
         sub_command = self.add_sub_command(
             subparser, "restart", 'Restart a storage node', usage='All functions and device drivers will be reset. '
                                   'During restart, the node does not accept IO. In a high-availability setup, '
                                   'this will not impact operations')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--max-lvol", help='Max lvol per storage node', dest='max_lvol', type=int, default=0)
         sub_command.add_argument("--max-snap", help='Max snapshot per storage node', dest='max_snap', type=int, default=0)
         sub_command.add_argument("--max-prov", help='Max provisioning size of all storage nodes', dest='max_prov', default="")
@@ -107,35 +106,35 @@ class CLIWrapper:
             subparser, "shutdown", 'Shutdown a storage node', usage='Once the command is issued, the node will stop accepting '
                                    'IO,but IO, which was previously received, will still be processed. '
                                    'In a high-availability setup, this will not impact operations.')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--force", help='Force node shutdown', required=False, action='store_true')
 
         # Suspend storage node
         sub_command = self.add_sub_command(
             subparser, "suspend", 'Suspend a storage node', usage='The node will stop accepting new IO, but will finish '
                                   'processing any IO, which has been received already.')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--force", help='Force node suspend', required=False, action='store_true')
 
         # Resume storage node
         sub_command = self.add_sub_command(subparser, "resume", 'Resume a storage node')
-        sub_command.add_argument("node_id", help='UUID of storage node')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         sub_command = self.add_sub_command(subparser, "get-io-stats", 'Get node IO statistics')
-        sub_command.add_argument("node_id", help='Node ID')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--history", help='list history records -one for every 15 minutes- '
                                                    'for XX days and YY hours -up to 10 days in total-, format: XXdYYh')
         sub_command.add_argument("--records", help='Number of records, default: 20', type=int, default=20)
 
         sub_command = self.add_sub_command(
             subparser, 'get-capacity', 'Get node capacity statistics')
-        sub_command.add_argument("node_id", help='Node ID')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument("--history", help='list history records -one for every 15 minutes- '
                                                    'for XX days and YY hours -up to 10 days in total-, format: XXdYYh')
 
         # List storage devices of the storage node
         sub_command = self.add_sub_command(subparser, "list-devices", 'List storage devices')
-        sub_command.add_argument("node_id", help='the node\'s UUID')
+        sub_command.add_argument("node_id", help='UUID of storage node').completer = self._completer_get_sn_list
         sub_command.add_argument(
             "-s", '--sort', help='Sort the outputs', required=False, nargs=1, choices=['node-seq', 'dev-seq', 'serial'])
         sub_command.add_argument(
@@ -220,7 +219,7 @@ class CLIWrapper:
 
         # check storage node
         sub_command = self.add_sub_command(subparser, "check", 'Health check storage node')
-        sub_command.add_argument("id", help='UUID of storage node')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         # check device
         sub_command = self.add_sub_command(subparser, "check-device", 'Health check device')
@@ -228,11 +227,11 @@ class CLIWrapper:
 
         # node info
         sub_command = self.add_sub_command(subparser, "info", 'Get node information')
-        sub_command.add_argument("id", help='Node UUID')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         # node info-spdk
         sub_command = self.add_sub_command(subparser, "info-spdk", 'Get SPDK memory information')
-        sub_command.add_argument("id", help='Node UUID')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         sub_command = self.add_sub_command(subparser, 'remove-jm-device', 'Remove JM device')
         sub_command.add_argument("jm_device_id", help='JM device ID')
@@ -243,17 +242,17 @@ class CLIWrapper:
         sub_command.add_argument("--force", help='Force device remove', required=False, action='store_true')
 
         sub_command = self.add_sub_command(subparser, 'send-cluster-map', 'send cluster map')
-        sub_command.add_argument("id", help='id')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         sub_command = self.add_sub_command(subparser, 'get-cluster-map', 'get cluster map')
-        sub_command.add_argument("id", help='id')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         sub_command = self.add_sub_command(subparser, 'make-primary',
                                            'In case of HA SNode, make the current node as primary')
-        sub_command.add_argument("id", help='id')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         sub_command = self.add_sub_command(subparser, 'dump-lvstore','Dump lvstore data')
-        sub_command.add_argument("id", help='id')
+        sub_command.add_argument("id", help='UUID of storage node').completer = self._completer_get_sn_list
 
         # check lvol
         #
@@ -336,8 +335,9 @@ class CLIWrapper:
 
         # Activate cluster
         sub_command = self.add_sub_command(subparser, 'activate', 'Create distribs and raid0 bdevs on all the storage node and move the cluster to active state')
-        sub_command.add_argument("cluster_id", help='the cluster UUID')
+        sub_command.add_argument("cluster_id", help='the cluster UUID').completer = self._completer_get_cluster_list
         sub_command.add_argument("--force", help='Force recreate distr and lv stores', required=False, action='store_true')
+        sub_command.add_argument("--force-lvstore-create", help='Force recreate lv stores', required=False, action='store_true', dest='force_lvstore_create')
 
         # show cluster list
         self.add_sub_command(subparser, 'list', 'Show clusters list')
@@ -345,11 +345,11 @@ class CLIWrapper:
         # show cluster info
         sub_command = self.add_sub_command(
             subparser, 'status', 'Show cluster status')
-        sub_command.add_argument("cluster_id", help='the cluster UUID')
+        sub_command.add_argument("cluster_id", help='the cluster UUID').completer = self._completer_get_cluster_list
 
         # show cluster info
         sub_command = self.add_sub_command(subparser, 'get', 'Show cluster info')
-        sub_command.add_argument("id", help='the cluster UUID')
+        sub_command.add_argument("id", help='the cluster UUID').completer = self._completer_get_cluster_list
 
         #sub_command = self.add_sub_command(
         #    subparser, 'suspend', 'Suspend cluster')
@@ -361,14 +361,14 @@ class CLIWrapper:
 
         sub_command = self.add_sub_command(
             subparser, 'get-capacity', 'Get cluster capacity')
-        sub_command.add_argument("cluster_id", help='the cluster UUID')
+        sub_command.add_argument("cluster_id", help='the cluster UUID').completer = self._completer_get_cluster_list
         sub_command.add_argument("--json", help='Print json output', required=False, action='store_true')
         sub_command.add_argument("--history", help='(XXdYYh), list history records (one for every 15 minutes) '
                                                    'for XX days and YY hours (up to 10 days in total).')
 
         sub_command = self.add_sub_command(
             subparser, 'get-io-stats', 'Get cluster IO statistics')
-        sub_command.add_argument("cluster_id", help='the cluster UUID')
+        sub_command.add_argument("cluster_id", help='the cluster UUID').completer = self._completer_get_cluster_list
         sub_command.add_argument("--records", help='Number of records, default: 20', type=int, default=20)
         sub_command.add_argument("--history", help='(XXdYYh), list history records (one for every 15 minutes) '
                                                    'for XX days and YY hours (up to 10 days in total).')
@@ -379,36 +379,38 @@ class CLIWrapper:
 
         # get-logs
         sub_command = self.add_sub_command(subparser, 'get-logs', 'Returns cluster status logs')
-        sub_command.add_argument("cluster_id", help='cluster uuid')
+        sub_command.add_argument("cluster_id", help='cluster uuid').completer = self._completer_get_cluster_list
 
         # get-secret
         sub_command = self.add_sub_command(subparser, 'get-secret', 'Get cluster secret')
-        sub_command.add_argument("cluster_id", help='cluster uuid')
+        sub_command.add_argument("cluster_id", help='cluster uuid').completer = self._completer_get_cluster_list
 
         # set-secret
         sub_command = self.add_sub_command(subparser, 'upd-secret', 'Updates the cluster secret')
-        sub_command.add_argument("cluster_id", help='cluster uuid')
+        sub_command.add_argument("cluster_id", help='cluster uuid').completer = self._completer_get_cluster_list
         sub_command.add_argument("secret", help='new 20 characters password')
 
         # check cluster
         sub_command = self.add_sub_command(subparser, "check", 'Health check cluster')
-        sub_command.add_argument("id", help='cluster UUID')
+        sub_command.add_argument("id", help='cluster UUID').completer = self._completer_get_cluster_list
 
         # update cluster
         sub_command = self.add_sub_command(subparser, "update", 'Update cluster mgmt services')
-        sub_command.add_argument("id", help='cluster UUID')
+        sub_command.add_argument("id", help='cluster UUID').completer = self._completer_get_cluster_list
 
         # graceful-shutdown storage nodes
         sub_command = self.add_sub_command(subparser, "graceful-shutdown", 'Graceful shutdown of storage nodes')
-        sub_command.add_argument("id", help='cluster UUID')
+        sub_command.add_argument("id", help='cluster UUID').completer = self._completer_get_cluster_list
 
         # graceful-startup storage nodes
         sub_command = self.add_sub_command(subparser, "graceful-startup", 'Graceful startup of storage nodes')
-        sub_command.add_argument("id", help='cluster UUID')
+        sub_command.add_argument("id", help='cluster UUID').completer = self._completer_get_cluster_list
+        sub_command.add_argument("--clear-data", help='clear Alceml data', dest='clear_data', action='store_true')
+        sub_command.add_argument("--spdk-image", help='SPDK image uri', dest='spdk_image')
 
         # get tasks list
         sub_command = self.add_sub_command(subparser, "list-tasks", 'List tasks by cluster ID')
-        sub_command.add_argument("cluster_id", help='UUID of the cluster')
+        sub_command.add_argument("cluster_id", help='UUID of the cluster').completer = self._completer_get_cluster_list
 
         # cancel task
         sub_command = self.add_sub_command(subparser, "cancel-task", 'Cancel task by ID')
@@ -418,7 +420,7 @@ class CLIWrapper:
         sub_command = self.add_sub_command(
             subparser, 'delete', 'Delete Cluster',
             usage="This is only possible, if no storage nodes and pools are attached to the cluster")
-        sub_command.add_argument("id", help='cluster UUID')
+        sub_command.add_argument("id", help='cluster UUID').completer = self._completer_get_cluster_list
 
 
         #
@@ -454,7 +456,10 @@ class CLIWrapper:
                                  default=0)
         sub_command.add_argument("--ha-type", help='LVol HA type (single, ha), default is cluster HA type',
                                  dest='ha_type', choices=["single", "ha", "default"], default='default')
-        sub_command.add_argument("--lvol-priority-class", help='Lvol priority class', type=int, choices=[0, 1], default=0)
+        sub_command.add_argument("--lvol-priority-class", help='Lvol priority class', type=int, default=0)
+        sub_command.add_argument("--namespace", help='Set LVol namespace for k8s clients')
+        sub_command.add_argument("--uid", help='Set LVol UUID')
+        sub_command.add_argument("--pvc_name", help='Set LVol PVC name for k8s clients')
 
 
         # set lvol params
@@ -696,7 +701,7 @@ class CLIWrapper:
 
         self.parser.add_argument("--cmd", help='cmd', nargs = '+')
 
-
+        argcomplete.autocomplete(self.parser)
 
     def init_parser(self):
         self.parser = argparse.ArgumentParser(prog=constants.SIMPLY_BLOCK_CLI_NAME, description='SimplyBlock management CLI')
@@ -801,7 +806,7 @@ class CLIWrapper:
                 ret = storage_ops.list_storage_nodes(args.json, args.cluster_id)
 
             elif sub_command == "remove":
-                ret = storage_ops.remove_storage_node(args.node_id, args.force_remove, args.force_migrate)
+                ret = storage_ops.remove_storage_node(args.node_id, args.force_remove)
 
             elif sub_command == "delete":
                 ret = storage_ops.delete_storage_node(args.node_id)
@@ -957,7 +962,7 @@ class CLIWrapper:
                 ret = self.cluster_add(args)
             elif sub_command == 'activate':
                 cluster_id = args.cluster_id
-                ret = cluster_ops.cluster_activate(cluster_id, args.force)
+                ret = cluster_ops.cluster_activate(cluster_id, args.force, args.force_lvstore_create)
             elif sub_command == 'status':
                 cluster_id = args.cluster_id
                 ret = cluster_ops.show_cluster(cluster_id)
@@ -1016,7 +1021,7 @@ class CLIWrapper:
                 ret = cluster_ops.cluster_grace_shutdown(args.id)
 
             elif sub_command == "graceful-startup":
-                ret = cluster_ops.cluster_grace_startup(args.id)
+                ret = cluster_ops.cluster_grace_startup(args.id, args.clear_data, args.spdk_image)
 
             elif sub_command == "delete":
                 ret = cluster_ops.delete_cluster(args.id)
@@ -1049,7 +1054,8 @@ class CLIWrapper:
                     max_size=max_size,
                     crypto_key1=args.crypto_key1,
                     crypto_key2=args.crypto_key2,
-                    lvol_priority_class=lvol_priority_class)
+                    lvol_priority_class=lvol_priority_class,
+                    uid=args.uid, pvc_name=args.pvc_name, namespace=args.namespace)
                 if results:
                     ret = results
                 else:
@@ -1264,17 +1270,13 @@ class CLIWrapper:
 
         print(ret)
 
-    def storage_node_list(self, args):
-        out = storage_ops.list_storage_nodes(self.db_store, args.json)
-        return out
-
     def storage_node_list_devices(self, args):
         node_id = args.node_id
         sort = args.sort
         if sort:
             sort = sort[0]
         is_json = args.json
-        out = storage_ops.list_storage_devices(self.db_store, node_id, sort, is_json)
+        out = storage_ops.list_storage_devices(node_id, sort, is_json)
         return out
 
     def cluster_add(self, args):
@@ -1404,12 +1406,17 @@ class CLIWrapper:
     def validate_cpu_mask(self, spdk_cpu_mask):
         return re.match("^(0x|0X)?[a-fA-F0-9]+$", spdk_cpu_mask)
 
+    def _completer_get_cluster_list(self, prefix, parsed_args, **kwargs):
+        db = db_controller.DBController()
+        return (cluster.get_id() for cluster in db.get_clusters() if cluster.get_id().startswith(prefix))
+
+
+    def _completer_get_sn_list(self, prefix, parsed_args, **kwargs):
+        db = db_controller.DBController()
+        return (cluster.get_id() for cluster in db.get_storage_nodes() if cluster.get_id().startswith(prefix))
+
 
 def main():
-    logger_handler = logging.StreamHandler()
-    logger_handler.setFormatter(logging.Formatter('%(asctime)s: %(levelname)s: %(filename)s:%(lineno)d: %(message)s'))
-    logger = logging.getLogger()
-    logger.addHandler(logger_handler)
 
     cli = CLIWrapper()
     cli.run()
