@@ -418,16 +418,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.mode = 'read-write'
     lvol.lvol_type = 'lvol'
     lvol.nqn = cl.nqn + ":lvol:" + lvol.uuid
-
-    lvol.ndcs = cl.distr_ndcs
-    lvol.npcs = cl.distr_npcs
-    lvol.distr_bs = cl.distr_bs
-    lvol.distr_chunk_bs = cl.distr_chunk_bs
     lvol.lvol_priority_class = lvol_priority_class
-    #lvol.distr_page_size = (distr_npcs+distr_npcs)*cl.page_size_in_blocks
-    lvol.cluster_size = lvol.ndcs * cl.page_size_in_blocks
-
-
 
     nodes = []
     if host_node:
@@ -836,6 +827,28 @@ def delete_lvol(id_or_name, force_delete=False):
 
     logger.debug(lvol)
     snode = db_controller.get_storage_node_by_id(lvol.node_id)
+
+    if not snode:
+        logger.error(f"lvol node id not found: {lvol.node_id}")
+        if not force_delete:
+            return False
+        lvol_events.lvol_delete(lvol)
+        lvol.remove(db_controller.kv_store)
+
+        # if lvol is clone and snapshot is deleted, then delete snapshot
+        if lvol.cloned_from_snap:
+            snap = db_controller.get_snapshot_by_id(lvol.cloned_from_snap)
+            if snap and snap.deleted is True:
+                lvols_count = 0
+                for lvol in db_controller.get_lvols():  # pass
+                    if lvol.cloned_from_snap == snap.get_id():
+                        lvols_count += 1
+                if lvols_count == 0:
+                    snapshot_controller.delete(snap.get_id())
+
+        logger.info("Done")
+        return True
+
     # creating RPCClient instance
     rpc_client = RPCClient(
         snode.mgmt_ip,
@@ -948,7 +961,7 @@ def delete_lvol(id_or_name, force_delete=False):
     # if lvol is clone and snapshot is deleted, then delete snapshot
     if lvol.cloned_from_snap:
         snap = db_controller.get_snapshot_by_id(lvol.cloned_from_snap)
-        if snap.deleted is True:
+        if snap and snap.deleted is True:
             lvols_count = 0
             for lvol in db_controller.get_lvols():  # pass
                 if lvol.cloned_from_snap == snap.get_id():
@@ -1028,8 +1041,8 @@ def list_lvols(is_json, cluster_id, pool_id_or_name, all=False):
     data = []
     for lvol in lvols:
         logger.debug(lvol)
-        # if lvol.deleted is True and all is False:
-        #     continue
+        if lvol.deleted is True and all is False:
+            continue
         data.append({
             "Id": lvol.uuid,
             "Name": lvol.lvol_name,
@@ -1037,7 +1050,6 @@ def list_lvols(is_json, cluster_id, pool_id_or_name, all=False):
             "Hostname": lvol.hostname,
             "HA": lvol.ha_type,
             "Priority": lvol.lvol_priority_class,
-            "Mod": f"{lvol.ndcs}x{lvol.npcs}",
             "Status": lvol.status,
             "IO Err": lvol.io_error,
             "Health": lvol.health_check,
