@@ -1,12 +1,12 @@
 import json
 
 import requests
-import logging
 
+from simplyblock_core import constants, utils
 from requests.adapters import HTTPAdapter
 from urllib3 import Retry
 
-logger = logging.getLogger()
+logger = utils.get_logger()
 
 
 def print_dict(d):
@@ -66,7 +66,8 @@ class RPCClient:
         if ret_code == 200:
             try:
                 data = response.json()
-                logger.debug("Response json: %s", json.dumps(data))
+                if method != "bdev_get_bdevs":
+                    logger.debug("Response json: %s", json.dumps(data))
             except Exception:
                 logger.debug("Response ret_content: %s", ret_content)
                 return ret_content, None
@@ -369,14 +370,14 @@ class RPCClient:
         params = {"name": name}
         return self._request2("ultra21_bdev_pass_delete", params)
 
-    def qos_vbdev_create(self, qos_bdev, base_bdev_name, base_nvme_hw_name, max_queue_size,
-                         inflight_io_threshold):
+    def qos_vbdev_create(self, qos_bdev, base_bdev_name, inflight_io_threshold):
         params = {
             "base_bdev_name": base_bdev_name,
             "name": qos_bdev,
-            "base_nvme_hw_name": base_nvme_hw_name,
-            "max_queue_size": max_queue_size,
-            "inflight_io_threshold": inflight_io_threshold
+            "max_num_queues": 2,
+            "standard_queue_weight": 3,
+            "low_priority_3_queue_weight": 1,
+            "inflight_io_threshold": inflight_io_threshold or 12
         }
 
         return self._request("qos_vbdev_create", params)
@@ -401,7 +402,7 @@ class RPCClient:
             # "use_scheduling": True,
             "use_optimized": True,
             "pba_nbalign": 4096,
-            "use_map_whole_page_on_1st_write": True
+            "use_map_whole_page_on_1st_write": False
         }
         if alceml_cpu_mask:
             params["bdb_lcpu_mask"] = int(alceml_cpu_mask, 16)
@@ -515,6 +516,9 @@ class RPCClient:
     def distr_add_nodes(self, params):
         return self._request("distr_add_nodes", params)
 
+    def distr_add_devices(self, params):
+        return self._request("distr_add_devices", params)
+
     def distr_status_events_update(self, params):
         # ultra/DISTR_v2/src_code_app_spdk/specs/message_format_rpcs__distrib__v5.txt#L396C1-L396C27
         return self._request("distr_status_events_update", params)
@@ -536,27 +540,6 @@ class RPCClient:
         }
         return self._request("bdev_nvme_attach_controller", params)
 
-    def bdev_nvme_attach_controller_tcp_JM(self, name, nqn, ip, port):
-        return self.bdev_nvme_attach_controller_tcp(name, nqn, ip, port)
-        # params = {
-        #     "name": name,
-        #     "trtype": "tcp",
-        #     "traddr": ip,
-        #     "adrfam": "ipv4",
-        #     "trsvcid": str(port),
-        #     "subnqn": nqn,
-        #     "fabrics_connect_timeout_us": 100000,
-        #
-        #     "num_io_queues": 16384,
-        #     "ctrlr_loss_timeout_sec": 0,
-        #     "multipath": "disable",
-        #     "reconnect_delay_sec": 0,
-        #
-        #     "fast_io_fail_timeout_sec": 1,
-        #
-        # }
-        # return self._request("bdev_nvme_attach_controller", params)
-
     def bdev_nvme_attach_controller_tcp_caching(self, name, nqn, ip, port):
         params = {
             "name": name,
@@ -618,7 +601,7 @@ class RPCClient:
             "reconnect_delay_sec": 1,
             "keep_alive_timeout_ms": 10000,
             "transport_ack_timeout": 9,
-            "timeout_us": 0
+            "timeout_us": constants.NVME_TIMEOUT_US
         }
         return self._request("bdev_nvme_set_options", params)
 
@@ -652,6 +635,11 @@ class RPCClient:
         else:
             return False
 
+    def accel_set_options(self):
+        params = {"small_cache_size": 512,
+                   "large_cache_size": 64}
+        return self._request("accel_set_options", params)
+
     def distr_status_events_get(self):
         return self._request("distr_status_events_get")
 
@@ -807,28 +795,34 @@ class RPCClient:
         params = {"id": app_thread_process_id, "cpumask": app_thread_mask}
         return self._request("thread_set_cpumask", params)
 
-    def distr_migration_to_primary_start(self, storage_ID, name):
+    def distr_migration_to_primary_start(self, storage_ID, name, qos_high_priority=False):
         params = {
             "name": name,
             "storage_ID": storage_ID,
         }
+        # if qos_high_priority:
+        #     params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_to_primary_start", params)
 
     def distr_migration_status(self, name):
         params = {"name": name}
         return self._request("distr_migration_status", params)
 
-    def distr_migration_failure_start(self, name, storage_ID):
+    def distr_migration_failure_start(self, name, storage_ID, qos_high_priority=False):
         params = {
             "name": name,
-            "storage_ID": storage_ID
+            "storage_ID": storage_ID,
         }
+        # if qos_high_priority:
+        #     params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_failure_start", params)
 
-    def distr_migration_expansion_start(self, name):
+    def distr_migration_expansion_start(self, name, qos_high_priority=False):
         params = {
             "name": name,
         }
+        # if qos_high_priority:
+        #     params["qos_high_priority"] = qos_high_priority
         return self._request("distr_migration_expansion_start", params)
 
     def bdev_raid_add_base_bdev(self, raid_bdev, base_bdev):
@@ -949,3 +943,41 @@ class RPCClient:
             "key_name": name
         }
         return self._request("accel_crypto_key_destroy", params)
+
+    def bdev_lvol_snapshot_register(self, lvol_name, snapshot_name, registered_uuid, blobid):
+        params = {
+            "lvol_name": lvol_name,
+            "snapshot_name": snapshot_name,
+            "blobid": blobid,
+            "registered_uuid": registered_uuid,
+        }
+        return self._request("bdev_lvol_snapshot_register", params)
+
+    def bdev_lvol_clone_register(self, clone_name, snapshot_name, registered_uuid, blobid):
+        params = {
+            "snapshot_name": snapshot_name,
+            "clone_name": clone_name,
+            "blobid": blobid,
+            "registered_uuid": registered_uuid,
+        }
+        return self._request("bdev_lvol_clone_register", params)
+
+    def distr_replace_id_in_map_prob(self, storage_ID_from, storage_ID_to):
+        params = {
+            "storage_ID_from": storage_ID_from,
+            "storage_ID_to": storage_ID_to,
+        }
+        return self._request("distr_replace_id_in_map_prob", params)
+
+    def nvmf_set_max_subsystems(self, max_subsystems):
+        params = {
+            "max_subsystems": max_subsystems,
+        }
+        return self._request("nvmf_set_max_subsystems", params)
+
+    def bdev_lvol_set_lvs_groupid(self, lvs_name, groupid):
+        params = {
+            "groupid": groupid,
+            "lvs_name": lvs_name,
+        }
+        return self._request("bdev_lvol_set_lvs_groupid", params)
